/*
PFEdemo1 - Probas

Copyright (C) 2002 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

import simu.*;

//utilise Proba et MoreProba

/**
 * Algorithme de simulation pour SimuLab.
 * Poisson Sample
 */

//	compilation: javac -target 1.1 -bootclasspath ~/Java/jdk1.1/JDKClasses.zip -extdirs "" -encoding ISO-8859-1 -classpath .:SimuLab.jar PoissonSample.java

public class PoissonSample extends SimuApplet {
	final static int NTIMES = 400;
	final static int tempo = 500; // un affichage / 0.5s
	Affichage affPlot;
	Affichage affPlotLive;
	double mu;
	int npts;
	long ct;
	int nt;
	double[] tmean;

    public PoissonSample() {
        // paramtres d'entre
        ParamIn muParam = new ParamIn();
        muParam.type = "nombre";
        muParam.label = "mu";
        muParam.titre = "";
        //muParam.unit = "m/s";
        muParam.acquisition = "champ";
        muParam.defaut = "5";
        addParamIn(muParam);
        
        ParamIn nptsParam = new ParamIn();
        nptsParam.type = "nombre";
        nptsParam.label = "npts";
        nptsParam.titre = "sample size";
        nptsParam.acquisition = "champ";
        nptsParam.defaut = "30";
        addParamIn(nptsParam);
        
        // paramtres de sortie
		// sample
        ParamOut sx = new ParamOut();
        sx.type = "nombre";
        sx.label = "sx";
        sx.titre = "";
        addParamOut(sx);
        ParamOut sy = new ParamOut();
        sy.type = "nombre";
        sy.label = "sy";
        sy.titre = "";
        addParamOut(sy);
        
		// mean
        ParamOut mx = new ParamOut();
        mx.type = "nombre";
        mx.label = "mx";
        addParamOut(mx);
        ParamOut my = new ParamOut();
        my.type = "nombre";
        my.label = "my";
        addParamOut(my);
		
		// infos (n itrations)
        ParamOut info = new ParamOut();
        info.type = "string";
        info.label = "info";
        addParamOut(info);
		
        // options d'affichage
		appletDoc("PoissonSample_doc.xml");
        
        // affichage "plot"
        affPlot = new Affichage();
        affPlot.type = "plot";
        affPlot.titre = "bruit de Poisson";
        String[][] pparams = {{"sx", "sy"}};
        affPlot.params = pparams;
		affPlot.histogramme = true;
		affPlot.effacer = true;
		affPlot.recadrer = true;
        addAffichage(affPlot);

        // affichage "plotlive"
        affPlotLive = new Affichage();
        affPlotLive.type = "plotlive";
        affPlotLive.titre = "moyenne";
        String[][] pparams2 = {{"sx", "sy"},{"mx","my"}};
        affPlotLive.params = pparams2;
		affPlotLive.effacer = true;
		affPlotLive.histogramme = true;
        addAffichage(affPlotLive);
		
        // affichage "tableau"
        Affichage affTab = new Affichage();
        affTab.type = "tableau";
        affTab.titre = "tableau";
        affTab.params = pparams;
        addAffichage(affTab);
    }
	
    // calcul complet d'un coup (utilis avec les affichages "plot" et "tableau")
    public ListeValeurs calcul(ListeValeurs in) throws SimuException {
        mu = in.lireDouble("mu");
        npts = in.lireEntier("npts");
		
		double[] box = MoreProba.bestBoxS(mu, npts);
		affPlot.xdebut = "" + box[0];
		affPlot.xfin = "" + box[1];
		affPlot.ydebut = "" + box[2];
		affPlot.yfin = "" + box[3];
		
		int[] valeurs = Proba.sPoisson(mu, npts);
        ListeValeurs out = new ListeValeurs();
		for (int i=0; i<npts; i++) {
			out.ajouterDouble("sx", i + 1);
			out.ajouterDouble("sy", valeurs[i]);
		}
		
		out.ajouterString("info", "");
		
        return out;
    }
	
    // calcul live (utilis avec affichage "plotlive")
    // initialisation
    public void initCalculLive(ListeValeurs in) throws SimuException {
        mu = in.lireDouble("mu");
        npts = in.lireEntier("npts");
		
		double[] box = MoreProba.bestBoxS(mu, npts);
		affPlotLive.xdebut = "" + box[0];
		affPlotLive.xfin = "" + box[1];
		affPlotLive.ydebut = "" + box[2];
		affPlotLive.yfin = "" + box[3];
		
		tmean = new double[npts];
		
		nt = 0;
		ct = System.currentTimeMillis();
    }
    
    // Calcul du temps t au temps t+1.
    public ListeValeurs calculLive() throws SimuException {
        if (nt >= NTIMES)
            return null; // fin du calcul
		nt++;
		
		int[] valeurs = Proba.sPoisson(mu, npts);
        ListeValeurs out = new ListeValeurs();
		for (int i=0; i<npts; i++) {
			out.ajouterDouble("sx", i+1);
			out.ajouterDouble("sy", valeurs[i]);
		}
		
		for (int i=0; i<npts; i++) {
			out.ajouterDouble("mx", i+1.2);
			tmean[i] = ((nt - 1)*tmean[i] + valeurs[i])/nt;
			out.ajouterDouble("my", tmean[i]);
		}
		
		out.ajouterString("info", "nb d'itrations: " + nt);
		
		// pour bloquer le programme pendant dt-tempo secondes
		long ct2 = System.currentTimeMillis();
		if (ct2 - ct < tempo) {
	        try {
	            Thread.currentThread().sleep(tempo - (ct2 - ct));
	        } catch (InterruptedException e) {}
			ct += tempo;
		} else
			ct = ct2;
		
        return out;
    }
}
