/*
PFEdemo1 - Probas

Copyright (C) 2002 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

import simu.*;

//utilise Proba et MoreProba

/**
 * Algorithme de simulation pour SimuLab.
 * RepPoisson
 */

//	compilation: javac -target 1.1 -bootclasspath ~/Java/jdk1.1/JDKClasses.zip -extdirs "" -encoding ISO-8859-1 -classpath .:SimuLab.jar RepPoisson.java

public class RepPoisson extends SimuApplet {
	Affichage affPlot;
	double mu;
	int npts;
	boolean distrib = true;
	boolean tcl;
	// variable/constantes pour l'affichage "plotlive":
	final static int NTIMES = 4000;
	final static int tempo = 500; // un affichage / 0.5s
	final static int nSampleMax = 4000;
	final static int nipa = 1; // nombre d'itrations par affichage
	Affichage affPlotLive;
	long ct;
	int nt;
	int nmin, nmax;
	double xmin, xmax;
	int[] tna;
	double[] tdx;
	double[] fdx;
	double[] tgx;
	double[] fgx;

    public RepPoisson() {
        // paramtres d'entre
        ParamIn muParam = new ParamIn();
        muParam.type = "nombre";
        muParam.label = "mu";
        muParam.titre = "";
        muParam.acquisition = "champ";
        muParam.defaut = "5";
        addParamIn(muParam);
        
        ParamIn nptsParam = new ParamIn();
        nptsParam.type = "nombre";
        nptsParam.label = "npts";
        nptsParam.titre = "sample size";
        nptsParam.acquisition = "champ";
        nptsParam.defaut = "30";
        addParamIn(nptsParam);
        
        ParamIn tclParam = new ParamIn();
        tclParam.type = "string";
        tclParam.label = "tcl";
        tclParam.titre = "Gauss limit";
        tclParam.acquisition = "case";
        tclParam.defaut = "on";
        addParamIn(tclParam);
        
        // paramtres de sortie
		
		// sample
        ParamOut sx = new ParamOut();
        sx.type = "nombre";
        sx.label = "sx";
        sx.titre = "";
        addParamOut(sx);
        ParamOut sy = new ParamOut();
        sy.type = "nombre";
        sy.label = "sy";
        sy.titre = "";
        addParamOut(sy);
		
		// fct de rpartition
        ParamOut fx = new ParamOut();
        fx.type = "nombre";
        fx.label = "fx";
        addParamOut(fx);
        ParamOut fy = new ParamOut();
        fy.type = "nombre";
        fy.label = "fy";
        addParamOut(fy);
        
		// Gauss limit
        ParamOut gx = new ParamOut();
        gx.type = "nombre";
        gx.label = "gx";
        addParamOut(gx);
        ParamOut gy = new ParamOut();
        gy.type = "nombre";
        gy.label  = "gy";
        gy.titre = "";
        addParamOut(gy);
        
		// infos (n itrations)
        ParamOut info = new ParamOut();
        info.type = "string";
        info.label = "info";
        addParamOut(info);
		
        // options d'affichage
		appletDoc("RepPoisson_doc.xml");
        
        // affichage "plot"
        affPlot = new Affichage();
        affPlot.type = "plot";
        affPlot.titre = "bruit de Poisson - fct de rpartition";
        String[][] pparams = {{"sx", "sy"},{"fx","fy"},{"gx","gy"}};
        affPlot.params = pparams;
		affPlot.connecter = true;
		affPlot.effacer = true;
		//affPlot.recadrer = true;
        addAffichage(affPlot);
        
        // affichage "plotlive"
        affPlotLive = new Affichage();
        affPlotLive.type = "plotlive";
        affPlotLive.titre = "convergence";
        //String[][] pparams = {{"sx", "sy"},{"fx","fy"},{"gx","gy"}};
        affPlotLive.params = pparams;
		affPlotLive.points = "none";
		affPlotLive.connecter = true;
		affPlotLive.effacer = true;
        addAffichage(affPlotLive);
    }
	
    public ListeValeurs calcul(ListeValeurs in) throws SimuException {
        mu = in.lireDouble("mu");
        npts = in.lireEntier("npts");
		tcl = "on".equals(in.lireString("tcl"));
		
		double[] box = MoreProba.bestBoxF(mu);
		double xmin = box[0];
		double xmax = box[1];
		double ymin = box[2];
		double ymax = box[3];
		affPlot.xdebut = "" + xmin;
		affPlot.xfin = "" + xmax;
		affPlot.ydebut = "" + ymin;
		affPlot.yfin = "" + ymax;
		
		int[] valeurs = Proba.sPoisson(mu, npts);
		
		double[] tx = new double[npts];
		double[] fx = new double[npts];
		
		Proba.hpsort(valeurs);
		
		for (int i=0; i<npts; i++)
			fx[i] = 0;
		for (int i=0; i<npts; i++)
			for (int j=i; j<npts; j++)
				fx[j] = fx[j] + 1.0/npts;
		
        ListeValeurs out = new ListeValeurs();
		out.ajouterDouble("sx", xmin);
		out.ajouterDouble("sy", 0);
		out.ajouterDouble("sx", valeurs[0]);
		out.ajouterDouble("sy", 0);
		for (int i=0; i<npts; i++) {
			out.ajouterDouble("sx", valeurs[i]);
			out.ajouterDouble("sy", fx[i]);
			
			if (i != npts-1) {
				out.ajouterDouble("sx", valeurs[i+1]);
				out.ajouterDouble("sy", fx[i]);
			}
		}
		out.ajouterDouble("sx", xmax);
		out.ajouterDouble("sy", fx[npts-1]);
		
		int nF = (int)xmax - (int)xmin + 1;
		
		if (distrib) {
			// Poisson repartition function
			tx = new double[nF];
			for (int i=0; i<nF; i++)
				tx[i] = Math.floor(xmin + i);
			fx = Proba.fPoisson(mu, tx);
			out.ajouterDouble("fx", xmin);
			out.ajouterDouble("fy", 0);
			out.ajouterDouble("fx", tx[0]);
			out.ajouterDouble("fy", 0);
			for (int i=0; i<nF; i++) {
				out.ajouterDouble("fx", tx[i]);
				out.ajouterDouble("fy", fx[i]);
				
				if (i != nF-1) {
					out.ajouterDouble("fx", tx[i+1]);
					out.ajouterDouble("fy", fx[i]);
				}
			}
			out.ajouterDouble("fx", xmax);
			out.ajouterDouble("fy", fx[nF-1]);
		}
		
		if (tcl) {
			// normal repartition function
			//nF = 30*nF;
			nF = 2*nF;
			tx = new double[nF];
			double dx = (xmax - xmin) / (nF - 1.0);
			
			for (int i=0; i<nF; i++)
				tx[i] = xmin + dx*i;
			fx = Proba.fnormal(mu, Math.sqrt(mu), tx);
			for (int i=0; i<nF; i++) {
				out.ajouterDouble("gx", tx[i]);
				out.ajouterDouble("gy", fx[i]);
			}
		} else { // pb affichage: comment ne pas avoir  mettre de valeur ici ?
			out.ajouterDouble("gx", 0);
			out.ajouterDouble("gy", 0);
		}
		
		out.ajouterString("info", "");
		
        return out;
    }
	
    // calcul live (utilis avec affichage "plotlive")
    // initialisation
    public void initCalculLive(ListeValeurs in) throws SimuException {
        mu = in.lireDouble("mu");
        npts = in.lireEntier("npts");
		tcl = "on".equals(in.lireString("tcl"));
		
		double[] boxF = MoreProba.bestBoxF(mu);
		xmin = boxF[0];
		xmax = boxF[1];
		double ymin = boxF[2];
		double ymax = boxF[3];
		
		affPlotLive.xdebut = "" + xmin;
		affPlotLive.xfin = "" + xmax;
		affPlotLive.ydebut = "" + ymin;
		affPlotLive.yfin = "" + ymax;
		
	    // Set Poisson Fx limits.
	    nmin = Math.max(0, (int)Math.floor(xmin));
	    nmax = Math.max(0, (int)Math.ceil(xmax));
		
		// Generates maximum Poisson sample once for all.
		tna = Proba.sPoisson(mu, nSampleMax);
		
		if (distrib) {
			// Poisson repartition function
			int nF = (int)nmax - (int)nmin + 1;
			tdx = new double[nF];
			for (int i=0; i<nF; i++)
				tdx[i] = Math.floor(nmin + i);
			fdx = Proba.fPoisson(mu, tdx);
		}
		
		if (tcl) {
			// normal repartition function
			//int nF = 30*(nmax-nmin+1);
			int nF = 3*(nmax-nmin+1);
			tgx = new double[nF];
			fgx = new double[nF];
			double dx = (xmax - xmin) / (nF - 1.0);
			
			for (int i=0; i<nF; i++)
				tgx[i] = xmin + dx*i;
			fgx = Proba.fnormal(mu, Math.sqrt(mu), tgx);
		}
		
		nt = 0;
		ct = System.currentTimeMillis();
    }
    
    // Calcul du temps t au temps t+1.
    public ListeValeurs calculLive() throws SimuException {
        if (nt >= NTIMES)
            return null; // fin du calcul
		
		double[] fx = new double[nt+1];
		for (int ii=0; ii<nipa; ii++) {
			nt++;
			Proba.iMerge1(tna, nt);
			fx = new double[nt];
			for (int i=0; i<nt; i++)
				fx[i] = 0;
			for (int i=0; i<nt; i++)
				fx[i] = (i+1.0)/nt;
		}
        ListeValeurs out = new ListeValeurs();
		out.ajouterDouble("sx", xmin);
		out.ajouterDouble("sy", 0);
		out.ajouterDouble("sx", tna[0]);
		out.ajouterDouble("sy", 0);
		for (int i=0; i<nt; i++) {
			if (i == nt - 1 || tna[i+1] > tna[i]) {
				out.ajouterDouble("sx", tna[i]);
				out.ajouterDouble("sy", fx[i]);
			}
			
			if (i != nt - 1 && tna[i+1] > tna[i]) {
				out.ajouterDouble("sx", tna[i+1]);
				out.ajouterDouble("sy", fx[i]);
			}
		}
		out.ajouterDouble("sx", xmax);
		out.ajouterDouble("sy", fx[nt-1]);
		
		if (distrib) {
			for (int i=0; i<tdx.length; i++) {
				if (i == tdx.length - 1 || tdx[i+1] > tdx[i]) {
					out.ajouterDouble("fx", tdx[i]);
					out.ajouterDouble("fy", fdx[i]);
				}
				
				if (i != tdx.length - 1 && tdx[i+1] > tdx[i]) {
					out.ajouterDouble("fx", tdx[i+1]);
					out.ajouterDouble("fy", fdx[i]);
				}
			}
		}
		if (tcl) {
			for (int i=0; i<tgx.length; i++) {
				out.ajouterDouble("gx", tgx[i]);
				out.ajouterDouble("gy", fgx[i]);
			}
		} else { // pb affichage: comment ne pas avoir  mettre de valeur ici ?
			out.ajouterDouble("gx", 0);
			out.ajouterDouble("gy", 0);
		}
		
		out.ajouterString("info", "nb d'itrations: " + nt);
		
		// pour bloquer le programme pendant dt-tempo secondes
		long ct2 = System.currentTimeMillis();
		if (ct2 - ct < tempo) {
	        try {
	            Thread.currentThread().sleep(tempo - (ct2 - ct));
	        } catch (InterruptedException e) {}
			ct += tempo;
		} else
			ct = ct2;
		
        return out;
    }
}
