/*
Flux2D - algorithme de simulation pour SimuApplet

Copyright (C) 2012 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformément aux dispositions de la Licence Publique Générale GNU, telle que publiée par la Free Software Foundation ; version 2 de la licence, ou encore (à votre choix) toute version ultérieure.

Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail, voir la Licence Publique Générale GNU .

Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même temps que ce programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

//utilise Proba et MoreProba

function isCanvasSupported() {
    var elem = document.createElement('canvas');
    return !!(elem.getContext && elem.getContext('2d'));
}

function versionApplet() {
    var applet = document.createElement('applet');
    var width, height; // espace disponible pour l'applet
    if (typeof(window.innerWidth) == 'number') {
        // autre que IE
        width = window.innerWidth;
        height = window.innerHeight;
    } else if ( document.documentElement && (document.documentElement.clientWidth || document.documentElement.clientHeight) ) {
        // IE 6+ en mode 'standards compliant'
        width = document.documentElement.clientWidth;
        height = document.documentElement.clientHeight;
    } else {
        width = '100%';
        height = '100%';
    }
    applet.setAttribute('width', width);
    applet.setAttribute('height', height);
    applet.setAttribute('archive', 'Probas.jar,SimuLab.jar');
    applet.setAttribute('code', 'Flux2D');

    document.body.appendChild(applet);
}

if (isCanvasSupported()) {
    new Flux2D();
} else {
    // on ne peut pas utiliser excanvas ou flashcanvas pour manipuler des bitmaps offscreen
    // donc on utilise la version sous forme d'applet en alternative pour les vieux navigateurs
    if (window.addEventListener)
        window.addEventListener('load', function() { versionApplet(); }, false);
    else if (window.attachEvent)
        window.attachEvent('onload', function() { versionApplet(); });
}

function Flux2D() {
    var that;
    
    this.npts = 128;
    this.lambda = 0.001;
    this.tmax = 100000;
    this.nomImage = 'Kolmogorov.png'; // comme dans l'applet Java
    this.lambdi = null; // valeurs de l'image initiale, int[][]
    this.w = 0;
    this.h = 0;
    this.vfact = 0.;
    this.ta = null; // double[][]
    this.ts = null; // double[][];
    this.moyenne = 0.;
    this.nt = 0;
    
    this.image = new Image();
    that = this;
    this.image.onload = function() {
        that.applet = new SimuApplet(that);
        that.applet.appletDoc('flux2d_doc.xml');
        that.applet.setDelaiIterations(100); // un affichage / 100ms
    };
    this.image.onerror = function(err) {
        alert("Erreur au chargement de l'image " + that.nomImage);
    };
    this.image.src = this.nomImage;
}

Flux2D.prototype.tableau2D = function(x, y) {
    var t, i, j;
    t = new Array(x);
    for (i=0; i<x; i++)
        t[i] = new Array(y);
    for (i=0; i<x; i++)
        for (j=0; j<y; j++)
            t[i][j] = 0;
    return(t);
}

Flux2D.prototype.lectureImage = function() {
    var img, ic, ctx, imgData, pixels, iy, ix, ip, red;
    
    img = this.image;
    this.w = img.width;
    this.h = img.height;
    ic = document.createElement('canvas');
    ic.width = this.w;
    ic.height = this.h;
    ctx = ic.getContext('2d');
    ctx.drawImage(img, 0, 0);
    imgData = ctx.getImageData(0, 0, this.w, this.h);
    pixels = imgData.data;
    
    this.lambdi = this.tableau2D(this.w, this.h);
    ip = 0;
    for (iy=0; iy<this.h; iy++) {
        for (ix=0; ix<this.w; ix++) {
            red = pixels[ip]; // on suppose que l'image est en niveaux de gris, donc inutile de convertir
            this.lambdi[ix][iy] = red;
            ip = ip + 4;
        }
    }
}

Flux2D.prototype.initCalculLive = function(lin) {
    var aff, vitesse, somme, i, j;
    
    if (this.lambdi == null)
        this.lectureImage();
    
    aff = this.applet.getAffichage('image');
    aff.imgdimx = this.w;
    aff.imgdimy = this.h;
    
    vitesse = lin.lireDouble('vitesse');
    somme = 0.;
    for (i=0; i<this.w; i++)
        for (j=0; j<this.h; j++)
            somme += this.lambdi[i][j];
    this.vfact = vitesse / somme;
    
    this.ta = this.tableau2D(this.w, this.h);
    this.ts = this.tableau2D(this.w, this.h);
    
    for (i=0; i<this.w; i++)
        for (j=0; j<this.h; j++)
            this.ta[i][j] = 0;
    
    this.moyenne = 0;
    
    this.nt = 0;
}

Flux2D.prototype.calculLive = function() {
    var mu, num, i, j, v, moy2, out;
    
    if (this.nt >= this.tmax)
        return null; // fin du calcul
    
    this.nt++;
    
    for (i=0; i<this.w; i++) {
        for (j=0; j<this.h; j++) {
            mu = this.lambdi[i][j] * this.vfact;
            num = Proba.sPoisson1(mu);
            this.ta[i][j] += num;
            this.moyenne += num / (this.h * this.w);
        }
    }
    
    moy2 = 128 / this.moyenne;
    for (i=0; i<this.w; i++) {
        for (j=0; j<this.h; j++) {
            v = this.ta[i][j] * moy2;
            if (v > 255)
                v = 255;
            this.ts[i][j] = v;
        }
    }
    
    out = new ListeValeurs();
    
    out.ajouterString('info', "nb d'itérations: " + this.nt + "  moyenne=" + this.moyenne.toFixed(3));
    
    out.ajouter('ts', this.ts);
    
    return out;
}
