/*
MoreProba

Copyright (C) 2012 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformément aux dispositions de la Licence Publique Générale GNU, telle que publiée par la Free Software Foundation ; version 2 de la licence, ou encore (à votre choix) toute version ultérieure.

Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail, voir la Licence Publique Générale GNU .

Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même temps que ce programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/


/**
 * Utilitaires pour les démos
 */
MoreProba = function() {

return {

    /**
    * best box for the repartition function
    * [0] : xmin
    * [1] : xmax
    * [2] : ymin
    * [3] : ymax
    */
    //public static double[] bestBoxF(double mu) {
    bestBoxF: function(mu) {
        var fact, box;
        fact = 3.5;
        box = new Array(4);
        
        box[0] = Math.max(-0.5, Math.floor(mu - fact*Math.sqrt(mu)));
        box[1] = Math.ceil(mu + fact*Math.sqrt(mu)) + 0.5;
        box[2] = 0.0;
        box[3] = 1.01;
        
        return(box);
    },

    /**
    * best box for the Sample
    * [0] : xmin
    * [1] : xmax
    * [2] : ymin
    * [3] : ymax
    */
    //public static double[] bestBoxS(double mu, int npts) {
    bestBoxS: function(mu, npts) {
        var box = new Array(4);
        
        box[0] = 0.5;
        box[1] = npts + 0.5;
        box[2] = Math.max(0.0, mu - 5.*Math.sqrt(mu));
        box[3] = Math.ceil(mu + 5.*Math.sqrt(mu)) + 0.1;
        
        return(box);
    },

    /**
    * best box for the Histogram
    * [0] : xmin
    * [1] : xmax
    * [2] : ymin
    * [3] : ymax
    */
    //public static double[] bestBoxH(double mu, int npts) {
    bestBoxH: function(mu, npts) {
        var fact, proba, box;
        
        fact = 3.5;
        box = new Array(4);
        
        box[0] = Math.max(-0.5, Math.floor(mu - 3.5*Math.sqrt(mu)));
        box[1] = Math.max(10., Math.ceil(mu + 3.5*Math.sqrt(mu))) + 0.5;
        box[2] = 0.0;
        try {
            proba = Proba.pPoisson(mu, Math.floor(mu));
        } catch (ex) {
            // on ignore l'erreur ?
            proba = 0;
        }
        box[3] = npts*proba + fact*Math.sqrt(npts*proba*(1.0 - proba));
        
        return(box);
    },

    /**
    * best box for PFEdemo2 (Erlang...)
    * [0] : xmin
    * [1] : xmax
    * [2] : ymin
    * [3] : ymax
    */
    //public static double[] bestBox2(double lambda, int k) {
    bestBox2: function(lambda, k) {
        var fact, box, nnu, sigma;
        
        fact = 3.5;
        box = new Array(4);
        
        nnu = k;
        sigma = Math.sqrt(nnu/(lambda*lambda));
        
        box[0] = Math.max(0.0, nnu/lambda - fact*sigma);
        box[1] = nnu/lambda + fact*sigma;
        box[2] = 0.0;
        box[3] = 1.001;
        
        return(box);
    },

    /**
    * valeur maximale d'un tableau d'entiers
    */
    //public static int maxval(int[] t) {
    maxval: function(t) {
        var max, i;
        
        max = t[0];
        for (i=0; i<t.length; i++)
            if (t[i] > max)
                max = t[i];
        return(max);
    },

    /**
    * valeur minimale d'un tableau d'entiers
    */
    //public static int minval(int[] t) {
    minval: function(t) {
        var min, i;
        
        min = t[0];
        for (i=0; i<t.length; i++)
            if (t[i] < min)
                min = t[i];
        return(min);
    },

    //public static double[] doHistoSample(double[] y, double ymin, double ymax, int nh) {
    doHistoSample: function(y, ymin, ymax, nh) {
        var histo, Dy, limit, histup, histdown, i, j;

        histo = new Array(nh);
        Dy = (ymax - ymin) / (nh - 1);
        histdown = 0;
        for (i=0; i<nh; i++) {
           limit = ymin + Dy*i;
           histup = 0;
           for (j=0; j<y.length; j++)
               if (y[j] <= limit)
                   histup++;
           histo[i] = histup - histdown;
           histdown = histup;
        }
        return(histo);
    }


};

}();
